from .noire import Noire

__version__ = "0.1.0"

__author__ = "Geoffrey Yu"
__email__ = "geoffxy@gmail.com"

__description__ = "Programmatic access to Mailman via its publicly-accessible web interface."
__url__ = "https://github.com/geoffxy/noire/"
